import re


def convert_to_dict(ispyb_object):
    d = {}

    if isinstance(ispyb_object, dict):
        d.update(ispyb_object)
    else:
        for key in ispyb_object.__keylist__:
            val = getattr(ispyb_object, key)
            if not isinstance(val, dict):
                val = convert_to_dict(val)
            elif isinstance(val, list):
                val = [
                    convert_to_dict(x) if not isinstance(x, dict) else x for x in val
                ]
            elif isinstance(val, dict):
                val = dict(
                    [
                        (k, convert_to_dict(x) if not isinstance(x, dict) else x)
                        for k, x in val.items()
                    ]
                )

            d[key] = val

    return d


def str_to_camel(name):
    if isinstance(name, str):
        components = name.split("_")
        # We capitalize the first letter of each component except the first one
        # with the 'title' method and join them together.
        name = components[0].lower() + "".join(x.title() for x in components[1:])

    return name


def str_to_snake(name):
    s = re.sub("(.)([A-Z][a-z]+)", r"\1_\2", name)
    return re.sub("([a-z0-9])([A-Z])", r"\1_\2", s).lower()


def _convert_dict_rec(fun, d, recurse=True):
    converted = {}

    for key, value in d.items():
        if isinstance(value, dict) and recurse:
            value = _convert_dict_rec(fun, value)

        converted[fun(key)] = value

    return converted


def to_camel(d):
    return _convert_dict_rec(str_to_camel, d)


def from_camel(d):
    return _convert_dict_rec(str_to_snake, d)



from mxcube3.core.adapter.adapter_base import ActuatorAdapterBase
from mxcube3.core.util.adapterutils import export
from mxcube3.core.models.adaptermodels import HOBeamModel, HOBeamValueModel


class BeamAdapter(ActuatorAdapterBase):
    def __init__(self, ho, *args, **kwargs):
        super(BeamAdapter, self).__init__(ho, *args, **kwargs)

    def limits(self):
        """ """
        return (-1, -1)

    def _get_aperture(self) -> tuple:
        """
        Returns list of apertures and the one currently used.

        :return: Tuple, (list of apertures, current aperture)
        :rtype: tuple
        """
        aperture_list, current_aperture = [], None
        beam_ho = self._ho

        aperture_list = beam_ho.get_available_size()["values"]
        current_aperture = beam_ho.get_value()[-1]

        return aperture_list, current_aperture

    def _get_value(self) -> HOBeamValueModel:
        beam_ho = self._ho

        beam_info_dict = {"position": [], "shape": "", "size_x": 0, "size_y": 0}
        sx, sy, shape, _label = beam_ho.get_value()

        if beam_ho is not None:
            beam_info_dict.update(
                {
                    "position": beam_ho.get_beam_position_on_screen(),
                    "size_x": sx,
                    "size_y": sy,
                    "shape": shape.value,
                }
            )

        aperture_list, current_aperture = self._get_aperture()

        beam_info_dict.update(
            {"apertureList": aperture_list, "currentAperture": current_aperture}
        )

        return HOBeamValueModel(**{"value": beam_info_dict})

    @export
    def get_size(self) -> HOBeamModel:
        pass

    @export
    def set_size(self, value: HOBeamModel) -> HOBeamModel:
        pass

    def data(self) -> HOBeamModel:
        return HOBeamModel(**self._dict_repr())



from decimal import Decimal

from mxcubecore.BaseHardwareObjects import HardwareObjectState

from mxcube3.core.adapter.adapter_base import ActuatorAdapterBase
from mxcube3.core.util.networkutils import RateLimited


class FluxAdapter(ActuatorAdapterBase):
    def __init__(self, ho, *args, **kwargs):
        """
        Args:
            (object): Hardware object.
        """
        super(FluxAdapter, self).__init__(ho, *args, **kwargs)

        self._read_only = ho.read_only

        try:
            ho.connect("valueChanged", self._value_change)
        except Exception:
            pass

    @RateLimited(6)
    def _value_change(self, value, **kwargs):
        value = "{:.2E}".format(Decimal(self._ho.get_value()))
        self.value_change(value, **kwargs)

    def _set_value(self, value=None):
        """Read only"""

    def _get_value(self):
        """
        Get the photon flux.
        Returns:
            (float as str): Flux.
        """
        try:
            # value = self._ho.current_flux
            value = "{:.2E}".format(Decimal(self._ho.get_value()))
        except Exception:
            value = "0"

        return value

    """
    def message(self):
        return ""
    """

    def limits(self):
        """No limits"""
        return (-1, -1)

    def state(self):
        """Always READY"""
        return HardwareObjectState.READY.name



from mxcubecore.BaseHardwareObjects import HardwareObjectState

from mxcube3.core.adapter.adapter_base import ActuatorAdapterBase
from mxcube3.core.models.adaptermodels import HOMachineInfoModel
from mxcube3.core.util.networkutils import RateLimited


class MachineInfoAdapter(ActuatorAdapterBase):
    def __init__(self, ho, *args, **kwargs):
        """
        Args:
            (object): Hardware object.
        """
        super(MachineInfoAdapter, self).__init__(ho, *args, **kwargs)
        ho.connect("valueChanged", self._value_change)
        self._unique = True

    def _set_value(self, value):
        pass

    @RateLimited(0.1)
    def _value_change(self, *args, **kwargs):
        self.value_change(self.get_value(), **kwargs)

    def _get_value(self) -> HOMachineInfoModel:
        return HOMachineInfoModel(
            **{
                "value": {
                    "current": self.get_current(),
                    "message": self.get_message(),
                    "fillmode": self.get_fill_mode(),
                }
            }
        )

    def get_message(self):
        try:
            message = self._ho.get_message()
        except (TypeError, AttributeError):
            message = ""

        return message

    def get_current(self):
        try:
            current = "{:.1f} mA".format(
                round(float(self._ho.get_current()), 1)
            )
        except (TypeError, AttributeError):
            current = "-1"

        return current

    def get_fill_mode(self):
        try:
            fmode = self._ho.get_fill_mode()
        except (TypeError, AttributeError):
            fmode = ""

        return fmode

    def limits(self):
        """
        Returns: The detector distance limits.
        """
        return (-1, -1)

    def stop(self):
        pass

    def state(self):
        return HardwareObjectState.READY.value

    def data(self) -> HOMachineInfoModel:
        return HOMachineInfoModel(**self._dict_repr())
